
from odoo import fields, models


class ProductCategory(models.Model):
    _inherit = 'product.category'

    avatax_category_id = fields.Many2one(string="Avatax Cloud Category")  # update field string only
    avatax_onprem_category_id = fields.Many2one(
        'product.avatax.category',
        string="Avatax Onprem Category",
        help="https://taxcode.avatax.avalara.com/",
    )

    def _get_avatax_category_id(self):
        avatax_category_type = self._context.get('avatax_category_type')
        # if on premise, get the specific "onprem" avatax category
        if avatax_category_type == 'onprem':
            categ = self
            while categ and not categ.avatax_onprem_category_id:
                categ = categ.parent_id
            if categ.avatax_onprem_category_id:
                return categ.avatax_onprem_category_id
        # otherwise if not on premise or no "onprem" avatax category are
        # configured, get the normal avatax category instead
        return super()._get_avatax_category_id()


class AccountExternalTaxMixin(models.AbstractModel):
    _inherit = 'account.external.tax.mixin'

    def _get_avatax_category_type_from_line_ref(self, model, res_id):
        """Get the hosting of the order related to the provided line reference"""
        hosting = ""
        if model == "sale.order.line":
            hosting = self.env["sale.order.line"].browse(int(res_id)).order_id.hosting
        elif model == "account.move.line":
            hosting = self.env["account.move.line"].browse(int(res_id)).hosting
        return "onprem" if hosting == "premise" else "online"

    def _get_avatax_invoice_line(self, line_data):
        """Create a `LineItemModel` based on line_data.

        :param line_data (dict): data returned by _get_line_data_for_external_taxes()
        """
        avatax_category_type = self._get_avatax_category_type_from_line_ref(line_data['model_name'], line_data['id'])
        if line_data['product_id']:
            line_data['product_id'] = line_data['product_id'].with_context(avatax_category_type=avatax_category_type)
        return super()._get_avatax_invoice_line(line_data)
